"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.relayNodeInterfacePlugin = void 0;
const nexus_1 = require("nexus");
const graphql_relay_1 = require("graphql-relay");
// Pretty much based on nodeDefinitions function from
//  relay: https://github.com/graphql/graphql-relay-js/blob/8f4ed1ad35805ef233ad9fd1af33abb9c0cad35a/src/node/node.js
function relayNodeInterfacePlugin(pluginConfig) {
    const { idFetcher, resolveType, idParser = graphql_relay_1.fromGlobalId, nonNullDefaults } = pluginConfig;
    if (!idFetcher) {
        throw new Error('idFetcher option is required for relayNodeInterfacePlugin');
    }
    if (typeof idFetcher !== 'function') {
        throw new Error('idFetcher option must be a function with signature: async ({ id, type }, ctx, info) => T where T is the resolved value for the ID');
    }
    if (!resolveType) {
        throw new Error('resolveType option is required for relayNodeInterfacePlugin');
    }
    if (typeof resolveType !== 'function') {
        throw new Error('resolveType option must be an function with signature async (value) => T where T is the type name or object');
    }
    return nexus_1.plugin({
        name: 'RelayNodeInterface',
        description: 'Creates the Relay Node interface and add node/nodes fields to the Query type',
        onInstall(builder) {
            if (!builder.hasType('Node')) {
                // node interface
                builder.addType(nexus_1.interfaceType({
                    name: 'Node',
                    description: 'An object with a global ID',
                    definition: (t) => {
                        ;
                        (nonNullDefaults ? (nonNullDefaults.output ? t.nonNull : t.nullable) : t).id('id', {
                            description: 'The global ID of the object.',
                        });
                        // overwrite the resolve type with the client function
                        // resolveType is just one way to map objs -> to the graphql type they represent
                        //  we could also have simply relied on isTypeOf on the GraphQLObjectType themselves
                        //  but as relay uses this by default, let's keep using it
                    },
                    resolveType,
                }));
                // node field
                builder.addType(nexus_1.queryField((t) => {
                    t.nullable.field('node', {
                        type: 'Node',
                        args: {
                            // defaults to nonNull to keep it backward compatible
                            id: (nonNullDefaults ? (nonNullDefaults.input ? nexus_1.nonNull : (v) => v) : nexus_1.nonNull)(nexus_1.idArg({
                                description: 'The global ID of an object',
                            })),
                        },
                        description: 'Fetches an object given its global ID',
                        resolve: (_obj, { id }, ctx, info) => idFetcher(graphql_relay_1.fromGlobalId(id), ctx, info),
                    });
                }));
                // nodes field
                builder.addType(nexus_1.queryField((t) => {
                    t.nonNull.list.field('nodes', {
                        type: 'Node',
                        args: {
                            ids: nexus_1.nonNull(nexus_1.list(nexus_1.nonNull(nexus_1.idArg({
                                description: 'The global IDs of objects',
                            })))),
                        },
                        description: 'Fetches objects given their global IDs',
                        resolve: (_obj, { ids }, ctx, info) => Promise.all(
                        // @ts-expect-error
                        ids.map((id) => Promise.resolve(idFetcher(idParser(id), ctx, info)))),
                    });
                }));
            }
        },
    });
}
exports.relayNodeInterfacePlugin = relayNodeInterfacePlugin;
//# sourceMappingURL=index.js.map